# Copyright (C) 2022 Leah Lackner
#
# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program.  If not, see <http://www.gnu.org/licenses/>.

import semantic_version  # type: ignore

__version__ = "1.2.2"

_v = semantic_version.Version(__version__)

__major_version__ = _v.major
__minor_version__ = _v.minor
__patch_version__ = _v.patch

__all__ = [
    "__version__",
    "__major_version__",
    "__minor_version__",
    "__patch_version__",
]
