""" Tool to organise your image gallery using machine learning.

Copyright (C) 2022 Leah Lackner

This program is free software: you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation, either version 3 of the License, or
(at your option) any later version.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program.  If not, see <http://www.gnu.org/licenses/>.

.. include:: ../README.md
"""
from cutyx import examples, exceptions, lib
from cutyx.__version__ import (
    __major_version__,
    __minor_version__,
    __patch_version__,
    __version__,
)

__author__ = "Leah Lackner"
__contact__ = "leah.lackner+github@gmail.com"
__copyright__ = "Copyright (C) 2022 Leah Lackner"
__deprecated__ = False
__email__ = "leah.lackner+github@gmail.com"
__license__ = "GPLv3"
__maintainer__ = "Leah Lackner"

__all__ = ["lib", "examples", "exceptions"]
